"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generatePackageJSON = exports.runGenerateCore = void 0;
const generate_1 = require("../service/builder/generate");
const path_1 = __importDefault(require("path"));
const webpack_1 = require("../service/builder/webpack");
const fs_extra_1 = __importDefault(require("fs-extra"));
const console_1 = require("../util/console");
const config_1 = require("../config");
const chalk_1 = __importDefault(require("chalk"));
const types_1 = require("../../types");
const lodash_template_1 = __importDefault(require("lodash.template"));
const util_1 = require("../util");
let lastDeps = null;
function isInLastDeps(deps) {
    for (const i in deps) {
        if (!lastDeps || !(lastDeps === null || lastDeps === void 0 ? void 0 : lastDeps[i]) || (lastDeps === null || lastDeps === void 0 ? void 0 : lastDeps[i]) !== deps[i]) {
            (0, console_1.log)(`package.json ${i}:${deps[i]} 和 上一次依赖${lastDeps === null || lastDeps === void 0 ? void 0 : lastDeps[i]} 不一致, 重新安装依赖...`);
            return false;
        }
    }
    return true;
}
function runGenerateCore(appBuildDir, appData, subAppDataList = [], dependencies = [], appKey, basename, buildTypeList, deployMode, runtime = types_1.RUNTIME.NONE, ignoreInstall = false, extraData = {
    isComposite: false,
    compProps: {},
}, domain) {
    return __awaiter(this, void 0, void 0, function* () {
        const timeTag = '-------------------- runGenerateCore';
        console.time(timeTag);
        const allAppDataList = subAppDataList.concat(appData);
        const deps = {};
        allAppDataList.forEach((app) => {
            Object.entries(app.npmDependencies).forEach(([name, version]) => {
                deps[name] = version;
            });
        });
        dependencies.forEach((compLb) => {
            if (compLb.isComposite && compLb.compLibCommonResource) {
                Object.assign(deps, compLb.compLibCommonResource.npm);
            }
        });
        yield Promise.all(allAppDataList.map((data) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const { pageInstanceList, rootPath = '' } = data;
            const appName = rootPath ? `Sub app ${rootPath}` : 'Main app';
            console.log(chalk_1.default.blue.bold(`Generating files for ${appName}`));
            const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `${rootPath}` : '');
            yield copy(['app/mountMpApis.js', 'app/mountAppApis.js', 'datasources'], dstDir);
            const tplStr = yield fs_extra_1.default.readFile(path_1.default.join(config_1.appTemplateDir, 'src/app/global-api.js'), {
                encoding: 'utf8',
            });
            const globalApiContent = (0, lodash_template_1.default)(tplStr, { interpolate: /<%=([\s\S]+?)%>/g })({
                appId: appKey,
                subPackageName: rootPath,
                domain,
                appConfig: (0, util_1.JsonToStringWithVariableName)({
                    id: appKey,
                    envId: appData.envId,
                    clientId: (_a = appData.extra) === null || _a === void 0 ? void 0 : _a.clientId,
                    loginConfigVersion: (_b = appData.extra) === null || _b === void 0 ? void 0 : _b.loginConfigVersion,
                    staticResourceDomain: domain || '',
                    envVersion: deployMode === types_1.DEPLOY_MODE.UPLOAD ? 'production' : 'preview',
                    pages: pageInstanceList
                        .sort((item) => (item.isHome ? -1 : 1))
                        .map((item) => {
                        var _a;
                        return ({
                            id: item.id,
                            type: ((_a = item.data) === null || _a === void 0 ? void 0 : _a.isCustomLoginPage) ? 'login' : '',
                        });
                    }),
                }),
            });
            yield fs_extra_1.default.writeFile(path_1.default.join(dstDir, 'app/global-api.js'), globalApiContent);
            yield (0, generate_1.generateAllPageJsxFile)(pageInstanceList, dstDir, rootPath, dependencies, extraData, buildTypeList);
            yield (0, generate_1.generateCodeFromTpl)(data, dstDir, dependencies, appKey, rootPath, deployMode, buildTypeList, extraData);
            yield (0, generate_1.writeLowCodeFiles)(data, dstDir);
        })));
        yield (0, generate_1.generateRouterFile)(allAppDataList, appBuildDir, basename, buildTypeList);
        yield (0, generate_1.generateAppStyleLessFile)(allAppDataList, appBuildDir);
        yield (0, generate_1.generateThemeVarsFile)(appData.themeVars, appBuildDir);
        if (lastDeps && isInLastDeps(deps)) {
            (0, console_1.notice)('package.json dependencies 已经安装，如出现未安装成功或找不到依赖，请重启wa watch');
        }
        else {
            yield generatePackageJSON(deps, appBuildDir, appKey);
            yield (0, webpack_1.installDependencies)(appBuildDir, {
                runtime,
                ignoreInstall,
            });
            lastDeps = deps;
        }
        console.timeEnd(timeTag);
    });
}
exports.runGenerateCore = runGenerateCore;
function generatePackageJSON(dependencies = {}, appBuildDir, appKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const packageInfo = fs_extra_1.default.readJSONSync(path_1.default.join(config_1.appTemplateDir, 'package.json'));
        packageInfo.dependencies = Object.assign(Object.assign({}, packageInfo.dependencies), dependencies);
        packageInfo.name = `WeDa-${appKey}`;
        const dstFilePath = path_1.default.join(appBuildDir, 'package.json');
        yield fs_extra_1.default.writeFile(dstFilePath, JSON.stringify(packageInfo, null, '\t'));
    });
}
exports.generatePackageJSON = generatePackageJSON;
function copy(srcFiles, dstDir) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const entry of srcFiles) {
            const dstFile = path_1.default.join(dstDir, entry);
            console.log(dstFile);
            yield fs_extra_1.default.copy(path_1.default.join(config_1.appTemplateDir, 'src', entry), dstFile);
        }
    });
}
